print("import: 'anaconda_mirror'")
import anaconda_mirror

print("import: 'anaconda_mirror.api_v4'")
import anaconda_mirror.api_v4

print("import: 'anaconda_mirror.api_v5'")
import anaconda_mirror.api_v5

print("import: 'anaconda_mirror.api_v6'")
import anaconda_mirror.api_v6

print("import: 'anaconda_mirror.api_local'")
import anaconda_mirror.api_local

print("import: 'anaconda_mirror.api_s3'")
import anaconda_mirror.api_s3

